#include "nrf_drv_spi.h"

volatile uint8_t tx_buf[10];
volatile bool transfer_in_progress = 0;
 
/**
 * @brief Work-around for transmitting 1 byte with SPIM.
 * @param spim: The SPIM instance that is in use.
 * @param ppi_channel: An unused PPI channel that will be used by the workaround.
 * @param gpiote_channel: An unused GPIOTE channel that will be used by the workaround.
 * @warning Must not be used when transmitting multiple bytes.
 * @warning After this workaround is used, the user must reset the PPI channel and the GPIOTE channel before attempting to transmit multiple bytes.
 */
void setup_workaround_for_ftpan_58(uint32_t ppi_channel, uint32_t gpiote_channel)
{
    // Create an event when SCK toggles.
    NRF_GPIOTE->CONFIG[gpiote_channel] = (GPIOTE_CONFIG_MODE_Event <<GPIOTE_CONFIG_MODE_Pos) | 
        (NRF_SPIM0->PSELSCK << GPIOTE_CONFIG_PSEL_Pos) | 
        (GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos);

    // Stop the spim instance when SCK toggles.
    NRF_PPI->CH[ppi_channel].EEP = (uint32_t)&NRF_GPIOTE->EVENTS_IN[gpiote_channel];
    NRF_PPI->CH[ppi_channel].TEP = (uint32_t)&NRF_SPIM0->TASKS_STOP;
    NRF_PPI->CHENSET = 1U << ppi_channel;

    // The spim instance cannot be stopped mid-byte, so it will finish
    // transmitting the first byte and then stop. Effectively ensuring
    // that only 1 byte is transmitted.
}

void spi_master_init(void)
{
	// Configure GPIO pins used for pselsck, pselmosi, pselmiso and pselss for SPI1
    nrf_gpio_cfg_output(SPI_PSELSCK1);
    nrf_gpio_cfg_output(SPI_PSELMOSI1);
    nrf_gpio_cfg_input(SPI_PSELMISO1, NRF_GPIO_PIN_PULLDOWN);
    nrf_gpio_cfg_output(SPI_PSELSS1);

    // Configure pins, frequency and mode 
    NRF_SPIM0->PSELSCK  = SPI_PSELSCK1;
    NRF_SPIM0->PSELMOSI = SPI_PSELMOSI1;
    NRF_SPIM0->PSELMISO = SPI_PSELMISO1;
    nrf_gpio_pin_set(SPI_PSELSS1);         // disable Set slave select (inactive high) 

    NRF_SPIM0->FREQUENCY = (uint32_t) ( 0x02000000UL << (uint32_t)Freq_8Mbps);
    NRF_SPIM0->CONFIG = (SPI_CONFIG_CPHA_Leading << SPI_CONFIG_CPHA_Pos) | 
    (SPI_CONFIG_CPOL_ActiveHigh << SPI_CONFIG_CPOL_Pos) | 
    (SPI_CONFIG_ORDER_MsbFirst << SPI_CONFIG_ORDER_Pos);
    NRF_SPIM0->ENABLE = (SPIM_ENABLE_ENABLE_Enabled << SPIM_ENABLE_ENABLE_Pos);
        
    NRF_SPIM0->INTENSET = 0;
    nrf_ppi_channel_t ppi_channel1;
    setup_workaround_for_ftpan_58(ppi_channel1,1);    
}


static void spi_master_rx2(const uint8_t *head, uint8_t head_size, uint8_t data_size, uint8_t *rx_data)
{
    uint8_t i = 0;
    
    for(i=0;i<head_size;i++)
    {
       tx_buf[i] =  *head++;
    }
    for(i=head_size;i<(head_size+data_size);i++)
    {
       tx_buf[i] =  0;
    }
   
    nrf_gpio_pin_clear(SPI_PSELSS1);
    
    NRF_SPIM0->EVENTS_END = 0U;
	NRF_SPIM0->EVENTS_STOPPED = 0U;
    
    NRF_SPIM0->TXD.MAXCNT = 1;  
    NRF_SPIM0->RXD.MAXCNT = 1;     
    NRF_SPIM0->TXD.LIST = 0;
    NRF_SPIM0->RXD.LIST = 0;
    NRF_SPIM0->SHORTS = SPIM_SHORTS_END_START_Msk;
    
    NRF_SPIM0->TXD.PTR = (uint32_t)tx_buf;
    NRF_SPIM0->TASKS_START = 1U;
    
    for(i=1;i<head_size;i++)
    {
        NRF_SPIM0->TXD.PTR = (uint32_t)tx_buf+i;
        
        while (NRF_SPIM0->EVENTS_END == 0U);
        NRF_SPIM0->EVENTS_END = 0U;
    } 
    NRF_SPIM0->RXD.PTR = (uint32_t)rx_data++;
    
    for(i=head_size;i<(head_size+data_size);i++)
    {
        NRF_SPIM0->TXD.PTR = (uint32_t)tx_buf+i;
        while (NRF_SPIM0->EVENTS_END == 0U);
        NRF_SPIM0->EVENTS_END = 0U;
        NRF_SPIM0->RXD.PTR = (uint32_t)rx_data++;  
    }         
    
    while (NRF_SPIM0->EVENTS_STARTED == 0U);
    NRF_SPIM0->EVENTS_STARTED = 0U;
    NRF_SPIM0->SHORTS = 0;
    
    while (NRF_SPIM0->EVENTS_END == 0U);    
    NRF_SPIM0->EVENTS_END = 0U; 
    
    nrf_gpio_pin_set(SPI_PSELSS1);
    NRF_SPIM0->TASKS_STOP = 1U;    
}

static void spi_master_tx2(const uint8_t *head, uint8_t head_size, uint8_t data_size, const uint8_t *tx_data)
{
    uint8_t i = 0;
    
    for(i=0;i<head_size;i++)
    {
       tx_buf[i] =  *head++;
    }
    for(i=head_size;i<(head_size+data_size);i++)
    {
       tx_buf[i] =  *tx_data++;
    }   
    
    nrf_gpio_pin_clear(SPI_PSELSS1);
    
    NRF_SPIM0->EVENTS_ENDTX = 0U;
	NRF_SPIM0->EVENTS_STOPPED = 0U;    
    NRF_SPIM0->TXD.MAXCNT = 1;
    NRF_SPIM0->SHORTS = SPIM_SHORTS_END_START_Msk;
    
    NRF_SPIM0->TXD.PTR = (uint32_t)tx_buf;
    NRF_SPIM0->TASKS_START = 1U;
    
    for(i=1;i<(head_size+data_size);i++)
    {
        NRF_SPIM0->TXD.PTR = (uint32_t)tx_buf+i;
        while (NRF_SPIM0->EVENTS_ENDTX == 0U);
        NRF_SPIM0->EVENTS_ENDTX = 0U;
    }         
    
    while (NRF_SPIM0->EVENTS_STARTED == 0U);
    NRF_SPIM0->EVENTS_STARTED = 0U;
    NRF_SPIM0->SHORTS = 0;
    
    while (NRF_SPIM0->EVENTS_ENDTX == 0U);    
    NRF_SPIM0->EVENTS_ENDTX = 0U;
   
    nrf_gpio_pin_set(SPI_PSELSS1);    
}

void mcu_asic_regs_read(uint16_t addr, uint8_t length, uint8_t *dat)
{
    uint8_t head[4];
    head[0] = (0xC0|(((uint8_t)(addr>>8))&0x0F));      // wr+add[11:8]
    head[1] = addr;
    head[2] = length;
    
    spi_master_rx2(head, 3, length, dat);
}

void mcu_asic_regs_write(uint16_t addr, uint8_t length, const uint8_t *dat)
{
    uint8_t head[3];
    head[0] = (0xD0|(((uint8_t)(addr>>8))&0x0F));      // wr+add[11:8]
    head[1] = addr;
    head[2] = length;
    
    spi_master_tx2(head, 3, length, dat);
}

 void mcu_asic_reg_write(uint16_t addr, const uint8_t dat)
{
    uint8_t i = 0;
    
    tx_buf[0] = (0xF0|(((uint8_t)(addr>>8))&0x0F));
    tx_buf[1] = addr;
    tx_buf[2] = dat;    
    
    nrf_gpio_pin_clear(SPI_PSELSS1);
    
    NRF_SPIM0->EVENTS_ENDTX = 0U;
	NRF_SPIM0->EVENTS_STOPPED = 0U;    
    NRF_SPIM0->TXD.MAXCNT = 1;
    NRF_SPIM0->SHORTS = SPIM_SHORTS_END_START_Msk;
    
    NRF_SPIM0->TXD.PTR = (uint32_t)tx_buf;
    NRF_SPIM0->TASKS_START = 1U;
    
    for(i=1;i<3;i++)
    {
        NRF_SPIM0->TXD.PTR = (uint32_t)tx_buf+i;
        while (NRF_SPIM0->EVENTS_ENDTX == 0U);
        NRF_SPIM0->EVENTS_ENDTX = 0U;
    }         
    
    while (NRF_SPIM0->EVENTS_STARTED == 0U);
    NRF_SPIM0->EVENTS_STARTED = 0U;
    NRF_SPIM0->SHORTS = 0;
    
    while (NRF_SPIM0->EVENTS_ENDTX == 0U);    
    NRF_SPIM0->EVENTS_ENDTX = 0U;
   
    nrf_gpio_pin_set(SPI_PSELSS1);        
}

void mcu_asic_img_read(uint32_t addr, uint8_t length, uint8_t *dat)
{
    uint8_t head[5];
    head[0] = (0x80|(((uint8_t)(addr>>16))&0x01));      // wr+add[11:8]
    head[1] = (addr>>8);
    head[2] = addr;
    
    head[3] = 0;
    head[4] = length;
    
    spi_master_rx2(head, 5, length, dat);
}
